#include "ivm.h"
#include <algorithm>
#include <stdexcept>
#define PICOJSON_USE_INT64
#define PICOJSON_USE_UINT64
#include <picojson.h>
result_t ivm_get_profile(device_t handle, char **buffer, void *(*allocate)(size_t))
{
	result_t result = result_ok;
	picojson::value::object profile_json;
	{
		ivm_measurement_settings_t command_output;
		picojson::value::object command_output_json;
		if(ivm_get_measurement_settings(handle, &command_output) != result_ok) result = result_error;
		{
			command_output_json["SamplingRate"]= picojson::value(static_cast<double>(command_output.SamplingRate));
		}
		{
			command_output_json["ProbeSignalFrequency"]= picojson::value(static_cast<double>(command_output.ProbeSignalFrequency));
		}
		{
			command_output_json["NumberPoints"]= picojson::value(static_cast<uint64_t>(command_output.NumberPoints));
		}
		{
			command_output_json["NumberChargePoints"]= picojson::value(static_cast<uint64_t>(command_output.NumberChargePoints));
		}
		{
			command_output_json["MaxVoltage"]= picojson::value(static_cast<double>(command_output.MaxVoltage));
		}
		{
			picojson::array field_output_json(4);
			for(unsigned int i = 0; i < 4; i++)
			    field_output_json[i] = picojson::value(static_cast<uint64_t>(command_output.Reserved0[i]));
			command_output_json["Reserved0"] = picojson::value(field_output_json);
		}
		{
			command_output_json["CurrentSensorMode"]= picojson::value(static_cast<uint64_t>(command_output.CurrentSensorMode));
		}
		{
			command_output_json["OutputMode"]= picojson::value(static_cast<uint64_t>(command_output.OutputMode));
		}
		{
			picojson::array field_output_json(19);
			for(unsigned int i = 0; i < 19; i++)
			    field_output_json[i] = picojson::value(static_cast<uint64_t>(command_output.Reserved1[i]));
			command_output_json["Reserved1"] = picojson::value(field_output_json);
		}
		profile_json["measurement_settings"] = picojson::value(command_output_json);
	}
	{
		ivm_calibration_settings_t command_output;
		picojson::value::object command_output_json;
		if(ivm_get_calibration_settings(handle, &command_output) != result_ok) result = result_error;
		{
			picojson::array field_output_json(8);
			for(unsigned int i = 0; i < 8; i++)
			    field_output_json[i] = picojson::value(static_cast<uint64_t>(command_output.Reserved0[i]));
			command_output_json["Reserved0"] = picojson::value(field_output_json);
		}
		{
			command_output_json["ADCMult"]= picojson::value(static_cast<double>(command_output.ADCMult));
		}
		{
			command_output_json["ADCVOffset"]= picojson::value(static_cast<double>(command_output.ADCVOffset));
		}
		{
			command_output_json["CurrentSense1Mult"]= picojson::value(static_cast<double>(command_output.CurrentSense1Mult));
		}
		{
			command_output_json["CurrentSense1Offset"]= picojson::value(static_cast<double>(command_output.CurrentSense1Offset));
		}
		{
			command_output_json["CurrentSense2Mult"]= picojson::value(static_cast<double>(command_output.CurrentSense2Mult));
		}
		{
			command_output_json["CurrentSense2Offset"]= picojson::value(static_cast<double>(command_output.CurrentSense2Offset));
		}
		{
			command_output_json["CurrentSense3Mult"]= picojson::value(static_cast<double>(command_output.CurrentSense3Mult));
		}
		{
			command_output_json["CurrentSense3Offset"]= picojson::value(static_cast<double>(command_output.CurrentSense3Offset));
		}
		{
			picojson::array field_output_json(72);
			for(unsigned int i = 0; i < 72; i++)
			    field_output_json[i] = picojson::value(static_cast<uint64_t>(command_output.Reserved1[i]));
			command_output_json["Reserved1"] = picojson::value(field_output_json);
		}
		profile_json["calibration_settings"] = picojson::value(command_output_json);
	}

	std::string out_str = picojson::value(profile_json).serialize(true);
	size_t size = out_str.size();
	(*buffer) = static_cast<char *>(allocate(size + 1));
	memcpy((*buffer), out_str.c_str(), size);
	(*buffer)[size] = '\0';
	return result;
}
result_t ivm_set_profile(device_t handle, char *buffer)
{
	result_t result = result_ok;
	picojson::value profile_json;
	try
	{
	    picojson::parse(profile_json, buffer);
	}
	catch(const std::exception &)
	{
	    return result_error;
	}
	if(!profile_json.is<picojson::object>())
	{
	    return result_error;
	}
	if(profile_json.contains("measurement_settings") && profile_json.get("measurement_settings").is<picojson::object>())
	{
		ivm_measurement_settings_t command_output;
		if(ivm_get_measurement_settings(handle, &command_output) != result_ok) result = result_error;
		picojson::value &command_output_json = profile_json.get("measurement_settings");
		if(command_output_json.contains("SamplingRate") && command_output_json.get("SamplingRate").is<double>()) 
		{
			double &field_output_json = command_output_json.get("SamplingRate").get<double>();
			command_output.SamplingRate = static_cast<float>(field_output_json);
		}
		else
		{
			result = result_error;
		}
		if(command_output_json.contains("ProbeSignalFrequency") && command_output_json.get("ProbeSignalFrequency").is<double>()) 
		{
			double &field_output_json = command_output_json.get("ProbeSignalFrequency").get<double>();
			command_output.ProbeSignalFrequency = static_cast<float>(field_output_json);
		}
		else
		{
			result = result_error;
		}
		if(command_output_json.contains("NumberPoints") && command_output_json.get("NumberPoints").is<uint64_t>()) 
		{
			uint64_t &field_output_json = command_output_json.get("NumberPoints").get<uint64_t>();
			command_output.NumberPoints = static_cast<uint32_t>(field_output_json);
		}
		else
		{
			result = result_error;
		}
		if(command_output_json.contains("NumberChargePoints") && command_output_json.get("NumberChargePoints").is<uint64_t>()) 
		{
			uint64_t &field_output_json = command_output_json.get("NumberChargePoints").get<uint64_t>();
			command_output.NumberChargePoints = static_cast<uint32_t>(field_output_json);
		}
		else
		{
			result = result_error;
		}
		if(command_output_json.contains("MaxVoltage") && command_output_json.get("MaxVoltage").is<double>()) 
		{
			double &field_output_json = command_output_json.get("MaxVoltage").get<double>();
			command_output.MaxVoltage = static_cast<float>(field_output_json);
		}
		else
		{
			result = result_error;
		}
		if(command_output_json.contains("Reserved0") && command_output_json.get("Reserved0").is<picojson::array>()) 
		{
			picojson::array &field_output_json = command_output_json.get("Reserved0").get<picojson::array>();
			if(field_output_json.size() == 4 && std::all_of(field_output_json.cbegin(), field_output_json.cend(), [](picojson::value const &e) { return e.is<uint64_t>(); })) 
			{
				unsigned int i;
				for(i=0; i < 4; i++) {
					command_output.Reserved0[i] = static_cast<uint8_t>(field_output_json[i].get<uint64_t>());
				}
			}
			else
			{
				result = result_error;
			}
		}
		else
		{
			result = result_error;
		}
		if(command_output_json.contains("CurrentSensorMode") && command_output_json.get("CurrentSensorMode").is<uint64_t>()) 
		{
			uint64_t &field_output_json = command_output_json.get("CurrentSensorMode").get<uint64_t>();
			command_output.CurrentSensorMode = static_cast<uint32_t>(field_output_json);
		}
		else
		{
			result = result_error;
		}
		if(command_output_json.contains("OutputMode") && command_output_json.get("OutputMode").is<uint64_t>()) 
		{
			uint64_t &field_output_json = command_output_json.get("OutputMode").get<uint64_t>();
			command_output.OutputMode = static_cast<uint8_t>(field_output_json);
		}
		else
		{
			result = result_error;
		}
		if(command_output_json.contains("Reserved1") && command_output_json.get("Reserved1").is<picojson::array>()) 
		{
			picojson::array &field_output_json = command_output_json.get("Reserved1").get<picojson::array>();
			if(field_output_json.size() == 19 && std::all_of(field_output_json.cbegin(), field_output_json.cend(), [](picojson::value const &e) { return e.is<uint64_t>(); })) 
			{
				unsigned int i;
				for(i=0; i < 19; i++) {
					command_output.Reserved1[i] = static_cast<uint8_t>(field_output_json[i].get<uint64_t>());
				}
			}
			else
			{
				result = result_error;
			}
		}
		else
		{
			result = result_error;
		}
		if(ivm_set_measurement_settings(handle, &command_output) != result_ok) result = result_error;
	}
	if(profile_json.contains("calibration_settings") && profile_json.get("calibration_settings").is<picojson::object>())
	{
		ivm_calibration_settings_t command_output;
		if(ivm_get_calibration_settings(handle, &command_output) != result_ok) result = result_error;
		picojson::value &command_output_json = profile_json.get("calibration_settings");
		if(command_output_json.contains("Reserved0") && command_output_json.get("Reserved0").is<picojson::array>()) 
		{
			picojson::array &field_output_json = command_output_json.get("Reserved0").get<picojson::array>();
			if(field_output_json.size() == 8 && std::all_of(field_output_json.cbegin(), field_output_json.cend(), [](picojson::value const &e) { return e.is<uint64_t>(); })) 
			{
				unsigned int i;
				for(i=0; i < 8; i++) {
					command_output.Reserved0[i] = static_cast<uint8_t>(field_output_json[i].get<uint64_t>());
				}
			}
			else
			{
				result = result_error;
			}
		}
		else
		{
			result = result_error;
		}
		if(command_output_json.contains("ADCMult") && command_output_json.get("ADCMult").is<double>()) 
		{
			double &field_output_json = command_output_json.get("ADCMult").get<double>();
			command_output.ADCMult = static_cast<float>(field_output_json);
		}
		else
		{
			result = result_error;
		}
		if(command_output_json.contains("ADCVOffset") && command_output_json.get("ADCVOffset").is<double>()) 
		{
			double &field_output_json = command_output_json.get("ADCVOffset").get<double>();
			command_output.ADCVOffset = static_cast<float>(field_output_json);
		}
		else
		{
			result = result_error;
		}
		if(command_output_json.contains("CurrentSense1Mult") && command_output_json.get("CurrentSense1Mult").is<double>()) 
		{
			double &field_output_json = command_output_json.get("CurrentSense1Mult").get<double>();
			command_output.CurrentSense1Mult = static_cast<float>(field_output_json);
		}
		else
		{
			result = result_error;
		}
		if(command_output_json.contains("CurrentSense1Offset") && command_output_json.get("CurrentSense1Offset").is<double>()) 
		{
			double &field_output_json = command_output_json.get("CurrentSense1Offset").get<double>();
			command_output.CurrentSense1Offset = static_cast<float>(field_output_json);
		}
		else
		{
			result = result_error;
		}
		if(command_output_json.contains("CurrentSense2Mult") && command_output_json.get("CurrentSense2Mult").is<double>()) 
		{
			double &field_output_json = command_output_json.get("CurrentSense2Mult").get<double>();
			command_output.CurrentSense2Mult = static_cast<float>(field_output_json);
		}
		else
		{
			result = result_error;
		}
		if(command_output_json.contains("CurrentSense2Offset") && command_output_json.get("CurrentSense2Offset").is<double>()) 
		{
			double &field_output_json = command_output_json.get("CurrentSense2Offset").get<double>();
			command_output.CurrentSense2Offset = static_cast<float>(field_output_json);
		}
		else
		{
			result = result_error;
		}
		if(command_output_json.contains("CurrentSense3Mult") && command_output_json.get("CurrentSense3Mult").is<double>()) 
		{
			double &field_output_json = command_output_json.get("CurrentSense3Mult").get<double>();
			command_output.CurrentSense3Mult = static_cast<float>(field_output_json);
		}
		else
		{
			result = result_error;
		}
		if(command_output_json.contains("CurrentSense3Offset") && command_output_json.get("CurrentSense3Offset").is<double>()) 
		{
			double &field_output_json = command_output_json.get("CurrentSense3Offset").get<double>();
			command_output.CurrentSense3Offset = static_cast<float>(field_output_json);
		}
		else
		{
			result = result_error;
		}
		if(command_output_json.contains("Reserved1") && command_output_json.get("Reserved1").is<picojson::array>()) 
		{
			picojson::array &field_output_json = command_output_json.get("Reserved1").get<picojson::array>();
			if(field_output_json.size() == 72 && std::all_of(field_output_json.cbegin(), field_output_json.cend(), [](picojson::value const &e) { return e.is<uint64_t>(); })) 
			{
				unsigned int i;
				for(i=0; i < 72; i++) {
					command_output.Reserved1[i] = static_cast<uint8_t>(field_output_json[i].get<uint64_t>());
				}
			}
			else
			{
				result = result_error;
			}
		}
		else
		{
			result = result_error;
		}
		if(ivm_set_calibration_settings(handle, &command_output) != result_ok) result = result_error;
	}

	return result;
}
